/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015, 2016. All Rights Reserved.
 * (c) Copyright HCL Technologies Ltd. 2016, 2019. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
var nh_port, activeTabId, tabInfo;
var cachedFtDoc = {};
var isBgResponse = false;
var CHROME_EXT_ID = "lblnfpdcingncbnkpodomelmnnhpkmbg"; // Extension ID from Chrome Web Store
var log = false;

function initVars() {
    nh_port = null;
    activeTabId = 0;
    tabInfo = { id: 0, status: "unknown" };
}

function loginfo(message) {
    console.log(message);
}

chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
    var type = request.type;
    switch (type) {
        case "nh_send":
            nh_sendMessage(request.result);
            break;
        case "update":
            isBgResponse = true;
            cachedFtDoc = request.value;
            if (log) loginfo("native_bg update isBgResponse: " + isBgResponse + ", cachedFtDoc: " + JSON.stringify(cachedFtDoc));
            break;
        case "tabinfo":
            tabInfo.id = sender.tab.id;
            tabInfo.status = request.tabStatus;
            if (log) loginfo("tabInfo.id - " + tabInfo.id + " : tabInfo.status - " + tabInfo.status);
            break;
        case "getScript":
            var res = "DontAppend";
            var JQUERY_JS = "jquery.js";
            var url = "http://localhost:" + config.portnumber + "/" + request.fileName;
            if (request.fileName == JQUERY_JS) {
                url = chrome.extension.getURL(JQUERY_JS);
            }
            var script = getRequest(url);
            if (chrome.runtime.id == CHROME_EXT_ID) {
                try {
                    chrome.tabs.executeScript(sender.tab.id, {
                        code: script
                    });
                } catch (e) {
                    res = "Append";
                    console.error("Exception in appending script " + request.fileName + " : " + e.message);
                }
            } else {
                res = "Append";
            }
            sendResponse({ result: res, script: script });
            break;
        case "init_natHost":
            if (!nh_port) {
                if (log) loginfo("initializing native messaging host");
                init();
            }
            break;
    }
});

function getRequest(url) {
    var xhr = new XMLHttpRequest();
    xhr.onload = function() {
        if (xhr.readyState == 4) {
            if (xhr.status == 200) {
                return xhr.responseText;
            } else {
                if (log) loginfo("xhr status failed " + xhr.status);
                return "";
            }
        }
    };
    xhr.open("GET", url, false);
    xhr.setRequestHeader("cache-control", "no-cache");
    try {
        xhr.send();
    } catch (err) {
        return "";
    }
    return xhr.responseText;
}

function nh_sendMessage(msg) {
    nh_port.postMessage(msg);
}

function cs_sendMessage(cs_isActive, activeTabId, msg) {
    if (cs_isActive && !isBgResponse) {
        chrome.tabs.sendMessage(activeTabId, msg);
    } else if (isBgResponse) {
        bgResponse(cs_isActive, activeTabId, msg);
    } else {
        nh_sendMessage({ retVal: null });
    }
}

function nh_onMessage(msg) {
    if (log) loginfo("nh_onMessage : " + JSON.stringify(msg));
    chrome.tabs.query({ active: true, currentWindow: true }, function(tabs) {
        activeTabId = tabs[0].id;
        // if(log) loginfo("activeTabId : " + activeTabId);
        var cs_isActive = true;
        if (activeTabId == tabInfo.id && tabInfo.status == "unloading") {
            cs_isActive = false;
        }
        if (cs_isActive) {
            cs_sendMessage(cs_isActive, activeTabId, msg);
        } else {
            var timerCounter = 0;
            var checkStatus = setInterval(function() {
                timerCounter += 1;
                if (activeTabId == tabInfo.id && tabInfo.status == "unloading") {
                    cs_isActive = false;
                }
                // if(log) loginfo(cs_isActive);
                if (cs_isActive || timerCounter > 10) {
                    clearInterval(checkStatus);
                    cs_sendMessage(cs_isActive, activeTabId, msg);
                }
            }, 50);
        }
    });
}

function nh_onDisconnect() {
    loginfo("nh_onDisconnect : " + chrome.runtime.lastError.message);
    initVars();
}

function nh_connect() {
    var hostName = "com.ibm.rft";
    loginfo("Connecting to native messaging host : " + hostName);
    nh_port = chrome.runtime.connectNative(hostName);
    nh_port.onMessage.addListener(nh_onMessage);
    nh_port.onDisconnect.addListener(nh_onDisconnect);
}

function init() {
    initVars();
    nh_connect();
}

function bgResponse(cs_isActive, activeTabId, msg) {
    if (msg.methodName == "isValidJSWrapper" || msg.methodName == "isAppActive") {
        if (log) loginfo("native_bg.bgResponse when msg.methodName is isValidJSWrapper, then follow normal flow");
        isBgResponse = false;
        cachedFtDoc = {};
        cs_sendMessage(cs_isActive, activeTabId, msg);
    } else {
        if (log) loginfo("native_bg.bgResponse");
        if (msg.methodName == "getDocumentHandle") {
            var handle = msg.methodArgs[0];
            if (handle == cachedFtDoc.doc.handle || handle == cachedFtDoc.browser.handle) {
                nh_sendMessage({ retVal: 0 });
            } else {
                nh_sendMessage({ retVal: cachedFtDoc.doc.handle });
            }
        } else if (msg.methodName == "getProperty") {
            var handle = msg.methodArgs[0];
            var prop = msg.methodArgs[1];
            if (handle == cachedFtDoc.doc.handle) {
                if (prop == "url" || prop == ".url") {
                    nh_sendMessage({ retVal: cachedFtDoc.doc.url });
                } else if (prop == "title" || prop == ".title") {
                    nh_sendMessage({ retVal: cachedFtDoc.doc.title });
                } else if (prop == "xpath" || prop == ".xpath") {
                    nh_sendMessage({ retVal: cachedFtDoc.doc.xpath });
                } else {
                    nh_sendMessage({ retVal: null });
                }
            } else {
                nh_sendMessage({ retVal: null });
            }
        } else if (msg.methodName == "getParent") {
            var handle = msg.methodArgs[0];
            if (handle == cachedFtDoc.doc.handle) {
                nh_sendMessage({ retVal: cachedFtDoc.browser.handle });
            } else {
                nh_sendMessage({ retVal: null });
            }
        } else {
            nh_sendMessage({ retVal: null });
        }
    }
}
